function [ T, Reperes ]=MGDref(q,Ksi,sigma)
% Calcule la position/orientation (T) de l'effecteur
% par rapport a son repere de base (0Tn selon
% le formalisme de Khalil-Kleinfinger)
% (et les positions/orientations des Reperes
% associes a chaque articulation) a partir
% des variables articulaires (q), du type
% de chaque articulation (sigma) et des parametres
% geometriques (Ksi) du robot.
% Ksi est un vecteur 6n contenant les parametres
% geometriques.
% Pour l'articulation i (transformation i-1Ti), i=1:n :
% si articulation rotoide (sigma(i)==0)
%   (beta_i, alpha_i, d_i, offset_i, K_i, r_i)
% si articulation prismatique (sigma(i)==1)
%    (beta_i, alpha_i, d_i, theta_i, K_i, offset_i)                   

nbArt=size(q,1);
if (nbArt~=size(sigma,1))|(6*nbArt~=size(Ksi,1))
   disp('Erreur de dimensions')
else
  T=eye(4,4); 
  rep=(nargout==2);
  if rep    
    Reperes=zeros(4*nbArt,4);  
  end
  for i=1:nbArt
    s=sigma(i);
    sbar=1-s;
    %disp(['*Rot(''y'',',num2str(Ksi(6*i-5)),')*Rot(''x'',',num2str(Ksi(6*i-4)),')...'])
    %disp(['*Trans(''x'',',num2str(Ksi(6*i-3)),')...'])
    %disp(['*Rot(''z'',',num2str(Ksi(6*i-2)+sbar*q(i)*Ksi(6*i-1)),')...'])
    %disp(['*Trans(''z'',',num2str(Ksi(6*i)+s*q(i)*Ksi(6*i-1)),')...'])
    
    T=T*Rot('y',Ksi(6*i-5))*Rot('x',Ksi(6*i-4))...
      *Trans('x',Ksi(6*i-3))...
      *Rot('z',Ksi(6*i-2)+sbar*(q(i)*Ksi(6*i-1)))...
      *Trans('z',Ksi(6*i)+s*(q(i)*Ksi(6*i-1)));
    % i.e. 
    % si prismatique :
    %  T=T*Rot('y',beta_i)*Rot('x',alpha_i)...
    %  *Trans('x',d_i)*Rot('z',theta_i)...
    %  *Trans('z',q(i)*K_i+offset_i);
    % si rotoide :
    %  T=T*Rot('y',beta_i)*Rot('x',alpha_i)...
    %  *Trans('x',d_i)*Rot('z',q(i)*K_i+offset_i)...
    %  *Trans('z',r_i);
    
    if rep    
      Reperes(4*i-3:4*i,:)=T;
    end
  end
end
if rep
  Reperes=[eye(4,4);Reperes];
end